@@ -17,17 +17,37 @@ import warnings
 
 from keystoneauth1 import discover
 import keystoneauth1.exceptions.catalog
+from keystoneauth1.loading import adapter as ks_load_adap
 from keystoneauth1 import session as ks_session
 import os_service_types
 import requestsexceptions
 from six.moves import urllib
+try:
+    import statsd
+except ImportError:
+    statsd = None
+try:
+    import prometheus_client
+except ImportError:
+    prometheus_client = None
+
 
-from openstack import _adapter
 from openstack import version as openstack_version
 from openstack import _log
 from openstack.config import _util
 from openstack.config import defaults as config_defaults
 from openstack import exceptions
+from openstack import proxy
+
+SCOPE_KEYS = {
+    'domain_id', 'domain_name',
+    'project_id', 'project_name',
+    'system_scope'
+}
+
+
+# Sentinel for nonexistence
+_ENOENT = object()
 
 
 def _make_key(key, service_type):
@@ -83,11 +103,77 @@ def from_session(session, name=None, region_name=None,
         app_name=app_name, app_version=app_version)
 
 
+def from_conf(conf, session=None, **kwargs):
+    """Create a CloudRegion from oslo.config ConfigOpts.
+
+    :param oslo_config.cfg.ConfigOpts conf:
+        An oslo.config ConfigOpts containing keystoneauth1.Adapter options in
+        sections named according to project (e.g. [nova], not [compute]).
+        TODO: Current behavior is to use defaults if no such section exists,
+        which may not be what we want long term.
+    :param keystoneauth1.session.Session session:
+        An existing authenticated Session to use. This is currently required.
+        TODO: Load this (and auth) from the conf.
+    :param kwargs:
+        Additional keyword arguments to be passed directly to the CloudRegion
+        constructor.
+    :raise openstack.exceptions.ConfigException:
+        If session is not specified.
+    :return:
+        An openstack.config.cloud_region.CloudRegion.
+    """
+    if not session:
+        # TODO(mordred) Fill this in - not needed for first stab with nova
+        raise exceptions.ConfigException("A Session must be supplied.")
+    config_dict = kwargs.pop('config', config_defaults.get_defaults())
+    stm = os_service_types.ServiceTypes()
+    for st in stm.all_types_by_service_type:
+        project_name = stm.get_project_name(st)
+        if project_name not in conf:
+            continue
+        opt_dict = {}
+        # Populate opt_dict with (appropriately processed) Adapter conf opts
+        try:
+            ks_load_adap.process_conf_options(conf[project_name], opt_dict)
+        except Exception:
+            # NOTE(efried): This is for oslo_config.cfg.NoSuchOptError, but we
+            # don't want to drag in oslo.config just for that.
+            continue
+        # Load them into config_dict under keys prefixed by ${service_type}_
+        for raw_name, opt_val in opt_dict.items():
+            config_name = '_'.join([st, raw_name])
+            config_dict[config_name] = opt_val
+    return CloudRegion(
+        session=session, config=config_dict, **kwargs)
+
+
 class CloudRegion(object):
     """The configuration for a Region of an OpenStack Cloud.
 
     A CloudRegion encapsulates the config information needed for connections
     to all of the services in a Region of a Cloud.
+
+    TODO(efried): Doc the rest of the kwargs
+
+    :param str region_name:
+        The default region name for all services in this CloudRegion. If
+        both ``region_name`` and ``config['region_name'] are specified, the
+        kwarg takes precedence. May be overridden for a given ${service}
+        via a ${service}_region_name key in the ``config`` dict.
+    :param dict config:
+        A dict of configuration values for the CloudRegion and its
+        services. The key for a ${config_option} for a specific ${service}
+        should be ${service}_${config_option}. For example, to configure
+        the endpoint_override for the block_storage service, the ``config``
+        dict should contain::
+
+            'block_storage_endpoint_override': 'http://...'
+
+        To provide a default to be used if no service-specific override is
+        present, just use the unprefixed ${config_option} as the service
+        key, e.g.::
+
+            'interface': 'public'
     """
     def __init__(self, name=None, region_name=None, config=None,
                  force_ipv4=False, auth_plugin=None,
@@ -96,10 +182,16 @@ class CloudRegion(object):
                  discovery_cache=None, extra_config=None,
                  cache_expiration_time=0, cache_expirations=None,
                  cache_path=None, cache_class='dogpile.cache.null',
-                 cache_arguments=None, password_callback=None):
+                 cache_arguments=None, password_callback=None,
+                 statsd_host=None, statsd_port=None, statsd_prefix=None,
+                 collector_registry=None):
         self._name = name
-        self.region_name = region_name
         self.config = _util.normalize_keys(config)
+        # NOTE(efried): For backward compatibility: a) continue to accept the
+        # region_name kwarg; b) make it take precedence over (non-service_type-
+        # specific) region_name set in the config dict.
+        if region_name is not None:
+            self.config['region_name'] = region_name
         self._extra_config = extra_config or {}
         self.log = _log.setup_logging('openstack.config')
         self._force_ipv4 = force_ipv4
@@ -116,6 +208,11 @@ class CloudRegion(object):
         self._cache_class = cache_class
         self._cache_arguments = cache_arguments
         self._password_callback = password_callback
+        self._statsd_host = statsd_host
+        self._statsd_port = statsd_port
+        self._statsd_prefix = statsd_prefix
+        self._statsd_client = None
+        self._collector_registry = collector_registry
 
         self._service_type_manager = os_service_types.ServiceTypes()
 
@@ -136,7 +233,8 @@ class CloudRegion(object):
     def __eq__(self, other):
         return (
             self.name == other.name
-            and self.region_name == other.region_name
+            # Ew
+            and self.get_region_name() == other.get_region_name()
             and self.config == other.config)
 
     def __ne__(self, other):
@@ -159,12 +257,13 @@ class CloudRegion(object):
         Always returns a valid string. It will have name and region_name
         or just one of the two if only one is set, or else 'unknown'.
         """
-        if self.name and self.region_name:
-            return ":".join([self.name, self.region_name])
-        elif self.name and not self.region_name:
+        region_name = self.get_region_name()
+        if self.name and region_name:
+            return ":".join([self.name, region_name])
+        elif self.name and not region_name:
             return self.name
-        elif not self.name and self.region_name:
-            return self.region_name
+        elif not self.name and region_name:
+            return region_name
         else:
             return 'unknown'
 
@@ -258,6 +357,12 @@ class CloudRegion(object):
             if st in config_dict:
                 return config_dict[st]
 
+    def get_region_name(self, service_type=None):
+        # If a region_name for the specific service_type is configured, use it;
+        # else use the one configured for the CloudRegion as a whole.
+        return self._get_config(
+            'region_name', service_type, fallback_to_unprefixed=True)
+
     def get_interface(self, service_type=None):
         return self._get_config(
             'interface', service_type, fallback_to_unprefixed=True)
@@ -297,6 +402,7 @@ class CloudRegion(object):
         return self._get_config('service_name', service_type)
 
     def get_endpoint(self, service_type):
+        auth = self.config.get('auth', {})
         value = self._get_config('endpoint_override', service_type)
         if not value:
             value = self._get_config('endpoint', service_type)
@@ -304,7 +410,13 @@ class CloudRegion(object):
             # If endpoint is given and we're using the none auth type,
             # then the endpoint value is the endpoint_override for every
             # service.
-            value = self.config.get('auth', {}).get('endpoint')
+            value = auth.get('endpoint')
+        if (not value and service_type == 'identity'
+                and SCOPE_KEYS.isdisjoint(set(auth.keys()))):
+            # There are a small number of unscoped identity operations.
+            # Specifically, looking up a list of projects/domains/system to
+            # scope to.
+            value = auth.get('auth_url')
         return value
 
     def get_connect_retries(self, service_type):
@@ -366,6 +478,7 @@ class CloudRegion(object):
                 verify=verify,
                 cert=cert,
                 timeout=self.config.get('api_timeout'),
+                collect_timing=self.config.get('timing'),
                 discovery_cache=self._discovery_cache)
             self.insert_user_agent()
             # Using old keystoneauth with new os-client-config fails if
@@ -438,19 +551,35 @@ class CloudRegion(object):
         # Seriously. Don't think about the existential crisis
         # that is the next line. You'll wind up in cthulhu's lair.
         service_type = self.get_service_type(service_type)
+        region_name = self.get_region_name(service_type)
         versions = self.get_session().get_all_version_data(
             service_type=service_type,
             interface=self.get_interface(service_type),
-            region_name=self.region_name,
+            region_name=region_name,
         )
-        region_versions = versions.get(self.region_name, {})
+        region_versions = versions.get(region_name, {})
         interface_versions = region_versions.get(
             self.get_interface(service_type), {})
         return interface_versions.get(service_type, [])
 
+    def _get_hardcoded_endpoint(self, service_type, constructor):
+        adapter = constructor(
+            session=self.get_session(),
+            service_type=self.get_service_type(service_type),
+            service_name=self.get_service_name(service_type),
+            interface=self.get_interface(service_type),
+            region_name=self.get_region_name(service_type),
+        )
+        endpoint = adapter.get_endpoint()
+        if not endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
+            if not endpoint.endswith('/'):
+                endpoint += '/'
+            endpoint = urllib.parse.urljoin(endpoint, 'v2.0')
+        return endpoint
+
     def get_session_client(
             self, service_type, version=None,
-            constructor=_adapter.OpenStackSDKAdapter,
+            constructor=proxy.Proxy,
             **kwargs):
         """Return a prepped keystoneauth Adapter for a given service.
 
@@ -467,44 +596,39 @@ class CloudRegion(object):
         """
         version_request = self._get_version_request(service_type, version)
 
+        kwargs.setdefault('region_name', self.get_region_name(service_type))
         kwargs.setdefault('connect_retries',
                           self.get_connect_retries(service_type))
         kwargs.setdefault('status_code_retries',
                           self.get_status_code_retries(service_type))
+        kwargs.setdefault('statsd_prefix', self.get_statsd_prefix())
+        kwargs.setdefault('statsd_client', self.get_statsd_client())
+        kwargs.setdefault('prometheus_counter', self.get_prometheus_counter())
+        kwargs.setdefault(
+            'prometheus_histogram', self.get_prometheus_histogram())
         endpoint_override = self.get_endpoint(service_type)
         version = version_request.version
         min_api_version = (
             kwargs.pop('min_version', None) or version_request.min_api_version)
         max_api_version = (
             kwargs.pop('max_version', None) or version_request.max_api_version)
+
         # Older neutron has inaccessible discovery document. Nobody noticed
         # because neutronclient hard-codes an append of v2.0. YAY!
-        if service_type == 'network':
+        # Also, older octavia has a similar issue.
+        if service_type in ('network', 'load-balancer'):
             version = None
             min_api_version = None
             max_api_version = None
             if endpoint_override is None:
-                network_adapter = constructor(
-                    session=self.get_session(),
-                    service_type=self.get_service_type(service_type),
-                    service_name=self.get_service_name(service_type),
-                    interface=self.get_interface(service_type),
-                    region_name=self.region_name,
-                )
-                network_endpoint = network_adapter.get_endpoint()
-                if not network_endpoint.rstrip().rsplit('/')[1] == 'v2.0':
-                    if not network_endpoint.endswith('/'):
-                        network_endpoint += '/'
-                    network_endpoint = urllib.parse.urljoin(
-                        network_endpoint, 'v2.0')
-                endpoint_override = network_endpoint
+                endpoint_override = self._get_hardcoded_endpoint(
+                    service_type, constructor)
 
         client = constructor(
             session=self.get_session(),
             service_type=self.get_service_type(service_type),
             service_name=self.get_service_name(service_type),
             interface=self.get_interface(service_type),
-            region_name=self.region_name,
             version=version,
             min_version=min_api_version,
             max_version=max_api_version,
@@ -571,6 +695,7 @@ class CloudRegion(object):
         if override_endpoint:
             return override_endpoint
 
+        region_name = self.get_region_name(service_type)
         service_name = self.get_service_name(service_type)
         interface = self.get_interface(service_type)
         session = self.get_session()
@@ -585,7 +710,7 @@ class CloudRegion(object):
             # the request
             endpoint = session.get_endpoint(
                 service_type=service_type,
-                region_name=self.region_name,
+                region_name=region_name,
                 interface=interface,
                 service_name=service_name,
                 **version_kwargs
@@ -595,12 +720,12 @@ class CloudRegion(object):
         if not endpoint:
             self.log.warning(
                 "Keystone catalog entry not found ("
-                "service_type=%s,service_name=%s"
+                "service_type=%s,service_name=%s,"
                 "interface=%s,region_name=%s)",
                 service_type,
                 service_name,
                 interface,
-                self.region_name,
+                region_name,
             )
         return endpoint
 
@@ -746,3 +871,61 @@ class CloudRegion(object):
     def get_concurrency(self, service_type=None):
         return self._get_service_config(
             'concurrency', service_type=service_type)
+
+    def get_statsd_client(self):
+        if not statsd:
+            return None
+        statsd_args = {}
+        if self._statsd_host:
+            statsd_args['host'] = self._statsd_host
+        if self._statsd_port:
+            statsd_args['port'] = self._statsd_port
+        if statsd_args:
+            return statsd.StatsClient(**statsd_args)
+        else:
+            return None
+
+    def get_statsd_prefix(self):
+        return self._statsd_prefix or 'openstack.api'
+
+    def get_prometheus_registry(self):
+        if not self._collector_registry and prometheus_client:
+            self._collector_registry = prometheus_client.REGISTRY
+        return self._collector_registry
+
+    def get_prometheus_histogram(self):
+        registry = self.get_prometheus_registry()
+        if not registry or not prometheus_client:
+            return
+        # We have to hide a reference to the histogram on the registry
+        # object, because it's collectors must be singletons for a given
+        # registry but register at creation time.
+        hist = getattr(registry, '_openstacksdk_histogram', None)
+        if not hist:
+            hist = prometheus_client.Histogram(
+                'openstack_http_response_time',
+                'Time taken for an http response to an OpenStack service',
+                labelnames=[
+                    'method', 'endpoint', 'service_type', 'status_code'
+                ],
+                registry=registry,
+            )
+            registry._openstacksdk_histogram = hist
+        return hist
+
+    def get_prometheus_counter(self):
+        registry = self.get_prometheus_registry()
+        if not registry or not prometheus_client:
+            return
+        counter = getattr(registry, '_openstacksdk_counter', None)
+        if not counter:
+            counter = prometheus_client.Counter(
+                'openstack_http_requests',
+                'Number of HTTP requests made to an OpenStack service',
+                labelnames=[
+                    'method', 'endpoint', 'service_type', 'status_code'
+                ],
+                registry=registry,
+            )
+            registry._openstacksdk_counter = counter
+        return counter
