@@ -21,7 +21,7 @@ import datetime
 import json
 import uuid
 
-from openstack.cloud._heat import template_format
+from openstack.orchestration.util import template_format
 from openstack.cloud import meta
 
 PROJECT_ID = '1c36b64c840a42cd9e9b931a369337f0'
@@ -64,6 +64,8 @@ def make_fake_flavor(flavor_id, name, ram=100, disk=1600, vcpus=24):
         u'swap': u'',
         u'vcpus': vcpus
     }
+
+
 FAKE_FLAVOR = make_fake_flavor(FLAVOR_ID, 'vanilla')
 FAKE_CHOCOLATE_FLAVOR = make_fake_flavor(
     CHOCOLATE_FLAVOR_ID, 'chocolate', ram=200)
@@ -218,12 +220,13 @@ def make_fake_stack_event(
 
 def make_fake_image(
         image_id=None, md5=NO_MD5, sha256=NO_SHA256, status='active',
-        image_name=u'fake_image'):
+        image_name=u'fake_image',
+        checksum=u'ee36e35a297980dee1b514de9803ec6d'):
     return {
         u'image_state': u'available',
         u'container_format': u'bare',
         u'min_ram': 0,
-        u'ramdisk_id': None,
+        u'ramdisk_id': 'fake_ramdisk_id',
         u'updated_at': u'2016-02-10T05:05:02Z',
         u'file': '/v2/images/' + image_id + '/file',
         u'size': 3402170368,
@@ -240,7 +243,7 @@ def make_fake_image(
         u'min_disk': 40,
         u'virtual_size': None,
         u'name': image_name,
-        u'checksum': u'ee36e35a297980dee1b514de9803ec6d',
+        u'checksum': checksum,
         u'created_at': u'2016-02-10T05:03:11Z',
         u'owner_specified.openstack.md5': NO_MD5,
         u'owner_specified.openstack.sha256': NO_SHA256,
